package com.yfd.monitor.storager.dao;

import com.yfd.monitor.gdw2019.bean.DeviceChannel;
import com.yfd.monitor.gdw2019.bean.GbStream;
import com.yfd.monitor.media.zlm.dto.StreamProxyItem;
import com.yfd.monitor.media.zlm.dto.StreamPushItem;
import com.yfd.monitor.service.bean.GPSMsgInfo;
import org.apache.ibatis.annotations.*;
import org.springframework.stereotype.Repository;

import java.util.List;

@Mapper
@Repository
public interface GbStreamMapper {

    @Insert("REPLACE INTO mon_gb_stream (app, stream, gbId, name, " +
            "longitude, latitude, streamType, mediaServerId, createTime) VALUES" +
            "(#{app}, #{stream}, #{gbId}, #{name}, " +
            "#{longitude}, #{latitude}, #{streamType}, " +
            "#{mediaServerId}, #{createTime})")
    @Options(useGeneratedKeys = true, keyProperty = "gbStreamId", keyColumn = "gbStreamId")
    int add(GbStream gbStream);

    @Update("UPDATE mon_gb_stream " +
            "SET app=#{app}," +
            "stream=#{stream}," +
            "gbId=#{gbId}," +
            "name=#{name}," +
            "streamType=#{streamType}," +
            "longitude=#{longitude}, " +
            "latitude=#{latitude}," +
            "mediaServerId=#{mediaServerId}" +
            "WHERE app=#{app} AND stream=#{stream}")
    int updateByAppAndStream(GbStream gbStream);

    @Update("UPDATE mon_gb_stream " +
            "SET app=#{app}," +
            "stream=#{stream}," +
            "gbId=#{gbId}," +
            "name=#{name}," +
            "streamType=#{streamType}," +
            "longitude=#{longitude}, " +
            "latitude=#{latitude}," +
            "mediaServerId=#{mediaServerId}" +
            "WHERE gbStreamId=#{gbStreamId}")
    int update(GbStream gbStream);

    @Delete("DELETE FROM mon_gb_stream WHERE app=#{app} AND stream=#{stream}")
    int del(String app, String stream);

    @Select("<script> "+
            "SELECT gs.* FROM mon_gb_stream gs " +
            "WHERE " +
            "1=1 " +
            " <if test='catalogId != null'> AND gs.gbStreamId in" +
            "(select pgs.gbStreamId from mon_platform_gb_stream pgs where pgs.platformId = #{platformId} and pgs.catalogId=#{catalogId})</if> " +
            " <if test='catalogId == null'> AND gs.gbStreamId not in" +
            "(select pgs.gbStreamId from mon_platform_gb_stream pgs where pgs.platformId = #{platformId}) </if> " +
            " <if test='query != null'> AND (gs.app LIKE concat('%',#{query},'%') OR gs.stream LIKE concat('%',#{query},'%') OR gs.gbId LIKE concat('%',#{query},'%') OR gs.name LIKE concat('%',#{query},'%'))</if> " +
            " <if test='mediaServerId != null' > AND gs.mediaServerId=#{mediaServerId} </if>" +
            " order by gs.gbStreamId asc " +
            "</script>")
    List<GbStream> selectAll(String platformId, String catalogId, String query, String mediaServerId);

    @Select("SELECT * FROM mon_gb_stream WHERE app=#{app} AND stream=#{stream}")
    GbStream selectOne(String app, String stream);

    @Select("SELECT * FROM mon_gb_stream WHERE gbId=#{gbId}")
    List<GbStream> selectByGBId(String gbId);

    @Select("SELECT gs.*, pgs.platformId as platformId, pgs.catalogId as catalogId FROM mon_gb_stream gs " +
            "LEFT JOIN mon_platform_gb_stream pgs ON gs.gbStreamId = pgs.gbStreamId " +
            "WHERE gs.gbId = #{gbId} AND pgs.platformId = #{platformId}")
    GbStream queryStreamInPlatform(String platformId, String gbId);

    @Select("<script> "+
            "select gt.gbId as channelId, gt.name, 'MonitorServer' as manufacture,  st.status, gt.longitude, gt.latitude, pc.id as parentId," +
            "       '1' as registerWay, pc.civilCode, 'live' as model, 'MonitorServer' as owner, '0' as parental,'0' as secrecy" +
            " from mon_gb_stream gt " +
            " left join (" +
            "    select " +
            " <if test='usPushingAsStatus != true'> sp.status as status, </if>" +
            " <if test='usPushingAsStatus == true'> sp.pushIng as status, </if>" +
            "sp.app, sp.stream from mon_stream_push sp" +
            "    union all" +
            "    select spxy.status, spxy.app, spxy.stream from mon_stream_proxy spxy" +
            " ) st on st.app = gt.app and st.stream = gt.stream" +
            " left join mon_platform_gb_stream pgs on  gt.gbStreamId = pgs.gbStreamId" +
            " left join mon_platform_catalog pc on pgs.catalogId = pc.id and pgs.platformId = pc.platformId" +
            " where pgs.platformId=#{platformId}" +
            "</script>")
    List<DeviceChannel> queryGbStreamListInPlatform(String platformId, boolean usPushingAsStatus);


    @Select("SELECT gs.* FROM mon_gb_stream gs LEFT JOIN mon_platform_gb_stream pgs " +
            "ON gs.gbStreamId = pgs.gbStreamId WHERE pgs.gbStreamId is NULL")
    List<GbStream> queryStreamNotInPlatform();

    @Delete("DELETE FROM mon_gb_stream WHERE streamType=#{type} AND gbId=NULL AND mediaServerId=#{mediaServerId}")
    void deleteWithoutGBId(String type, String mediaServerId);

    @Delete("<script> "+
            "DELETE FROM mon_gb_stream where " +
            "<foreach collection='streamProxyItemList' item='item' separator='or'>" +
            "(app=#{item.app} and stream=#{item.stream}) " +
            "</foreach>" +
            "</script>")
    void batchDel(List<StreamProxyItem> streamProxyItemList);

    @Delete("<script> "+
            "DELETE FROM mon_gb_stream where " +
            "<foreach collection='gbStreams' item='item' separator='or'>" +
            "(app=#{item.app} and stream=#{item.stream}) " +
            "</foreach>" +
            "</script>")
    void batchDelForGbStream(List<GbStream> gbStreams);

    @Insert("<script> " +
            "INSERT IGNORE into mon_gb_stream " +
            "(app, stream, gbId, name, " +
            "longitude, latitude, streamType, mediaServerId, createTime)" +
            "values " +
            "<foreach collection='subList' index='index' item='item' separator=','> " +
            "(#{item.app}, #{item.stream}, #{item.gbId}, #{item.name}, " +
            "#{item.longitude}, #{item.latitude}, #{item.streamType}, " +
            "#{item.mediaServerId}, #{item.createTime}) "+
            "</foreach> " +
            "</script>")
    @Options(useGeneratedKeys = true, keyProperty = "gbStreamId", keyColumn = "gbStreamId")
    void batchAdd(List<StreamPushItem> subList);

    @Update({"<script>" +
            "<foreach collection='gpsMsgInfos' item='item' separator=';'>" +
            " UPDATE" +
            " mon_gb_stream" +
            " SET longitude=#{item.lng}, latitude=#{item.lat} " +
            "WHERE gbId=#{item.id}"+
            "</foreach>" +
            "</script>"})
    int updateStreamGPS(List<GPSMsgInfo> gpsMsgInfos);

    @Select("<script> "+
                   "SELECT * FROM mon_gb_stream where " +
                   "<foreach collection='streamPushItems' item='item' separator='or'>" +
                   "(app=#{item.app} and stream=#{item.stream}) " +
                   "</foreach>" +
                   "</script>")
    List<GbStream> selectAllForAppAndStream(List<StreamPushItem> streamPushItems);

    @Update("UPDATE mon_gb_stream " +
            "SET mediaServerId=#{mediaServerId}" +
            "WHERE app=#{app} AND stream=#{stream}")
    void updateMediaServer(String app, String stream, String mediaServerId);

    @Update("<script> "+
                " <foreach collection='list' item='item' index='index' separator=';'>"+
                    "UPDATE mon_gb_stream " +
                    " SET name=#{item.name},"+
                    " gbId=#{item.gbId}"+
                    " WHERE app=#{item.app} and stream=#{item.stream}"+
                "</foreach>"+
            "</script>")
    int updateGbIdOrName(List<StreamPushItem> streamPushItemForUpdate);

    @Select("SELECT status FROM mon_stream_proxy WHERE app=#{app} AND stream=#{stream}")
    Boolean selectStatusForProxy(String app, String stream);

    @Select("SELECT status FROM mon_stream_push WHERE app=#{app} AND stream=#{stream}")
    Boolean selectStatusForPush(String app, String stream);

}
